# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2009- Spyder Kernels Contributors
#
# Licensed under the terms of the MIT License
# (see spyder_kernels/__init__.py for details)
# -----------------------------------------------------------------------------

"""
Tests for dochelpers.py
"""

# Standard library imports
import os
import sys

# Test library imports
import pytest

# Local imports
from spyder_kernels.utils.dochelpers import (
    getargtxt, getargspecfromtext, getdoc, getobj, getsignaturefromtext,
    isdefined)
from spyder_kernels.py3compat import PY2


class Test(object):
    def method(self, x, y=2):
        pass


@pytest.mark.skipif(
    PY2 or os.name == 'nt', reason="Only works on Linux and Mac")
@pytest.mark.skipif(
    sys.platform == 'darwin' and sys.version_info[:2] == (3, 8),
    reason="Fails on Mac with Python 3.8")
def test_dochelpers():
    """Test dochelpers."""
    assert getargtxt(Test.method) == ['x, ', 'y=2']
    assert not getargtxt(Test.__init__)

    assert getdoc(sorted) == {
        'note': 'Function of builtins module',
        'argspec': '(...)',
        'docstring': 'Return a new list containing all items from the '
                     'iterable in ascending order.\n\nA custom key function '
                     'can be supplied to customize the sort order, and the\n'
                     'reverse flag can be set to request the result in '
                     'descending order.',
        'name': 'sorted'
    }
    assert not getargtxt(sorted)

    assert isdefined('numpy.take', force_import=True)
    assert isdefined('__import__')
    assert not isdefined('zzz', force_import=True)

    assert getobj('globals') == 'globals'
    assert not getobj('globals().keys')
    assert getobj('+scipy.signal.') == 'scipy.signal'
    assert getobj('4.') == '4'


@pytest.mark.skipif(PY2, reason="Fails in Python 2")
def test_no_signature():
    """
    Test that we can get documentation for objects for which Python can't get a
    signature directly because it gives an error.

    This is a regression test for issue spyder-ide/spyder#21148
    """
    import numpy as np
    doc = getdoc(np.where)
    signature = doc['argspec']
    assert signature and signature != "(...)" and signature.startswith("(")
    assert doc['docstring']


@pytest.mark.parametrize(
    'text, name, expected',
    [
         ('foo(x, y)', 'foo', '(x, y)'),
         ('foo(x, y)', '', '(x, y)'),
    ]
)
def test_getsignaturefromtext_py2(text, name, expected):
    assert getsignaturefromtext(text, name) == expected


@pytest.mark.skipif(PY2, reason="Don't work in Python 2")
@pytest.mark.parametrize(
    'text, name, expected',
    [
         # Simple text with and without name
         ('foo(x, y)', 'foo', '(x, y)'),
         ('foo(x, y)', '', '(x, y)'),
         # Single arg
         ('foo(x)', '', '(x)'),
         ('foo(x = {})', '', '(x = {})'),
         # Not a valid identifier
         ('1a(x, y)', '', ''),
         # Valid identifier
         ('a1(x, y=2)', '', '(x, y=2)'),
         # Unicode identifier with and without name
         ('ΣΔ(x, y)', 'ΣΔ', '(x, y)'),
         ('ΣΔ(x, y)', '', '(x, y)'),
         # Multiple signatures in a single line
         ('ΣΔ(x, y) foo(a, b)', '', '(x, y)'),
         ('1a(x, y) foo(a, b)', '', '(a, b)'),
         # Multiple signatures in multiple lines
         ('foo(a, b = 1)\n\nΣΔ(x, y=2)', '', '(a, b = 1)'),
         ('1a(a, b = 1)\n\nΣΔ(x, y=2)', '', '(x, y=2)'),
         # Signature after math operations
         ('2(3 + 5) 3*(99) ΣΔ(x, y)', '', '(x, y)'),
         # No identifier
         ('(x, y)', '', ''),
         ('foo (a=1, b = 2)', '', ''),
         # Empty signature
         ('foo()', '', ''),
         ('foo()', 'foo', ''),
    ]
)
def test_getsignaturefromtext(text, name, expected):
    assert getsignaturefromtext(text, name) == expected


def test_multisignature():
    """
    Test that we can get at least one signature from an object with multiple
    ones declared in its docstring.
    """
    def foo():
        """
        foo(x, y) foo(a, b)
        foo(c, d)
        """

    signature = getargspecfromtext(foo.__doc__)
    assert signature == "(x, y)"


def test_multiline_signature():
    """
    Test that we can get signatures splitted into multiple lines in a
    docstring.
    """
    def foo():
        """
        foo(x,
            y)

        This is a docstring.
        """

    signature = getargspecfromtext(foo.__doc__)
    assert signature.startswith("(x, ")


if __name__ == "__main__":
    pytest.main()



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2009- Spyder Kernels Contributors
#
# Licensed under the terms of the MIT License
# (see spyder_kernels/__init__.py for details)
# -----------------------------------------------------------------------------

"""Tests for the User Module Reloader."""

# Stdlib imports
import os
import sys

# Third party imports
import pytest

# Local imports
from spyder_kernels.py3compat import to_text_string
from spyder_kernels.customize.umr import UserModuleReloader


@pytest.fixture
def user_module(tmpdir):
    """Create a simple module in tmpdir as an example of a user module."""
    if to_text_string(tmpdir) not in sys.path:
        sys.path.append(to_text_string(tmpdir))

    def create_module(modname):
        modfile = tmpdir.mkdir(modname).join('bar.py')
        code = """
def square(x):
    return x**2
        """
        modfile.write(code)

        init_file = tmpdir.join(modname).join('__init__.py')
        init_file.write('#')

    return create_module


def test_umr_skip_cython(user_module):
    """
    Test that the UMR doesn't try to reload modules when Cython
    support is active.
    """
    # Create user module
    user_module('foo')

    # Activate Cython support
    os.environ['SPY_RUN_CYTHON'] = 'True'

    # Create UMR
    umr = UserModuleReloader()

    import foo
    assert umr.is_module_reloadable(foo, 'foo') == False

    # Deactivate Cython support
    os.environ['SPY_RUN_CYTHON'] = 'False'


def test_umr_run(user_module):
    """Test that UMR's run method is working correctly."""
    # Create user module
    user_module('foo1')

    # Activate verbose mode in the UMR
    os.environ['SPY_UMR_VERBOSE'] = 'True'

    # Create UMR
    umr = UserModuleReloader()

    from foo1.bar import square
    umr.run()
    umr.modnames_to_reload == ['foo', 'foo.bar']


def test_umr_previous_modules(user_module):
    """Test that UMR's previos_modules is working as expected."""
    # Create user module
    user_module('foo2')

    # Create UMR
    umr = UserModuleReloader()

    import foo2
    assert 'IPython' in umr.previous_modules
    assert 'foo2' not in umr.previous_modules


def test_umr_namelist():
    """Test that the UMR skips modules according to its name."""
    umr = UserModuleReloader()

    assert umr.is_module_in_namelist('tensorflow')
    assert umr.is_module_in_namelist('pytorch')
    assert umr.is_module_in_namelist('spyder_kernels')
    assert not umr.is_module_in_namelist('foo')


def test_umr_reload_modules(user_module):
    """Test that the UMR only tries to reload user modules."""
    # Create user module
    user_module('foo3')

    # Create UMR
    umr = UserModuleReloader()

    # Don't reload stdlib modules
    import xml
    assert not umr.is_module_reloadable(xml, 'xml')

    # Don't reload third-party modules
    import numpy
    assert not umr.is_module_reloadable(numpy, 'numpy')

    # Reload user modules
    import foo3
    assert umr.is_module_reloadable(foo3, 'foo3')
